# -*- coding: utf-8 -*-

#######################################################################
# Copyright © 2007 Yuri Malheiros.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published
# the Free Software Foundation; version 2 only.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#######################################################################

"""
This module implements a class for responsible for manage all
opened text files of TextFlow.
"""

import gtk
import gtksourceview2
import os
import pango
import gettext

from TextFlow.widgets.TFSourceView import TFSourceView
from TextFlow.core.PreferencesManager import PreferencesManager
from TextFlow.core.TriggerManager import TriggerManager
from TextFlow.com.Search import Search
from TextFlow.core import constants

from TextFlow.widgets.FileDialog import *
from TextFlow.widgets.CloseAlertDialog import CloseAlertDialog
from TextFlow.widgets.ErrorDialog import ErrorDialog

_ = gettext.gettext

class DocumentManager(gtk.Notebook):
    """
    This class extend the gtk.Notebook widget with functions
    to manipulate opened files.
    """

    def __init__(self, interface):
        """
        Constructor.
        """
        super(DocumentManager, self).__init__()
        self.main_window = interface.main_window
        self.interface = interface
#        self.opened_files = []
        self.preferences_manager = PreferencesManager()
        self.__get_color_styles()
        self.search_functions = Search(self)
        
        close_button_style = ''' 
            style 'close_button' {
                xthickness = 0
                ythickness = 0
            }
            widget '*.close_button' style 'close_button'
        '''
        gtk.rc_parse_string(close_button_style)
        
        self.set_scrollable(True)
        self.connect("switch-page", self.change_tab)
        self.trigger_manager = TriggerManager(self)
        
    def __get_color_styles(self):
        """
        This method gets all available color styles.
        """
        styles_dir = os.getenv("HOME") + "/.textflow/colors"
        self.style_manager = gtksourceview2.StyleSchemeManager()
        self.style_manager.prepend_search_path(styles_dir)
    
    def change_tab(self, widget, page, page_num):
        """
        This method does some operations when tabs are changed.
        """
        previous_view = widget.get_active_view()
        if previous_view != None:
            previous_view.word_complete.stop_indexer()
        
        view = self.get_nth_page(page_num).get_child()
        
        view.word_complete.start_indexer()
        
        self.search_functions.view = view
        self.search_functions.buffer = view.buffer
        
        self.search_functions.cursor_mark = \
        self.search_functions.buffer.get_insert()
         
        title = view.get_file_uri()
        if title == "":
            file_name = constants.MESSAGE_0001
            direc = ""
        else:
            f_list = title.split("/")
            file_name = f_list.pop()
            direc = " (" + "/".join(f_list) + ")"
        
        self.main_window.set_title(str(file_name) + direc)
        begin = view.buffer.get_start_iter()
        end = view.buffer.get_end_iter()
        self.search_functions.update_matches(begin, end)
        
        pos = view.get_cursor_position()
        self.interface.statusbar.set_line_column(pos[0], pos[1])
        
    
#    def __check_encode(self, encode):
#        sub_menu = self.interface.menuitem_encoding.get_submenu()
#        sub_menu_children = sub_menu.get_children()
#        
#        for i in sub_menu_children:
#            sub_i = i.get_submenu()
#            for j in sub_i.get_children():
#                if encode == j.get_child().get_text():
#                    print j.get_child().get_text()
#                    j.set_active(True)
    
    def __create_gsv(self, file_uri, encode):
        """
        Create a TFSourceView and set some properties.
        
        @param file_uri: A file that will be displayed in TFSourceView.
        @type file_uri: A string
        
        @return: The TFSourceView created.
        @rtype: A TFSourceView object.
        """
#        tag_table = gtksourceview.SourceTagTable()
#        buffer = gtksourceview.SourceBuffer(tag_table)
        buffer = gtksourceview2.Buffer()
        pm = self.preferences_manager
        
        buffer.create_tag("searchmatch", background="lightblue")
        buffer.create_tag("sticky", background="#ff6100")
#        language = gtksourceview2.Language()
        buffer.set_highlight_syntax(True)
        view = TFSourceView(buffer, file_uri, encode)
        view.show()

        #applying properties
        self.set_font(view, pm.get_value("font"))
        self.set_tab_width(view, pm.get_value("indentation/tab_width"))
        self.set_use_spaces(view, pm.get_value("indentation/use_spaces"))
        self.set_auto_indent(view, pm.get_value("indentation/automatic"))
        self.set_show_right_margin(view, pm.get_value("right_margin/display"))
        self.set_right_margin_position(view, pm.get_value("right_margin/position"))
        self.set_show_line_numbers(view, pm.get_value("line_numbers"))
        self.set_highlight_current_line(view, pm.get_value("highlight_current_line"))
        self.set_text_wrapping(view, pm.get_value("text_wrapping"))
        self.set_brackets_matching(view, pm.get_value("brackets_matching"))
        view.set_smart_home_end(False)
        self.set_style(view, pm.get_value("color_style"))
        
        view.set_tab_width(4)
        view.set_auto_indent(True)
        view.set_wrap_mode(gtk.WRAP_WORD)
        
        view.connect('key-press-event', self.pre_key_press)
#        view.connect('event-after', self.pos_key_press)
        view.connect('key-release-event', self.key_release)
        view.buffer.connect("notify::cursor-position", self.cursor_moved, view,
                            self.interface.statusbar)
        
        return view
    
    def cursor_moved(self, buffer, position, view, statusbar):
        pos = view.get_cursor_position()
        statusbar.set_line_column(pos[0], pos[1])
    
    def key_release(self, widget, event):
        """
        This method does some operations when a key is realeased.
        """
        last = self.trigger_manager.last_shortcut
#         print "--------------------------"
#         print event.state
#         print event.keyval
#         print "--------------------------"
        if last in self.trigger_manager.sticky_keys:
            if event.state & gtk.gdk.CONTROL_MASK and event.state & gtk.gdk.SHIFT_MASK:
                pass
#                 print "ctrl+shift+" + unichr(event.keyval).lower()
            elif event.state & gtk.gdk.CONTROL_MASK:
                if "ctrl" in last and (event.keyval == 65507 or not (unichr(event.keyval) in self.trigger_manager.sticky_keys[last])):
                    try:
                        self.trigger_manager.sticky_shortcuts[last]()
                    #except KeyError:
                    #    return
                    finally:
                        self.trigger_manager.last_shortcut = None
    #         elif event.state & gtk.gdk.SHIFT_MASK:
    #             shortcut = "shift+" + unichr(event.keyval).lower()
    #             shortcut2 = unichr(event.keyval).lower()
    #             if shortcut in self.trigger_manager.sticky_shortcuts:
    #                 self.trigger_manager.sticky_shortcuts[shortcut]()
    #             elif shortcut2 in self.trigger_manager.sticky_shortcuts:
    #                 self.trigger_manager.sticky_shortcuts[shortcut2]()
            else:
                shortcut = unichr(event.keyval).lower()
                last_key = last.split("+")[-1]
                if shortcut == last_key:
                    try:
                        self.trigger_manager.sticky_shortcuts[last]()
                    #except KeyError:
                    #    return
                    finally:
                        self.trigger_manager.last_shortcut = None
        
    def pre_key_press(self, widget, event):
#        print event.keyval
#        print event.state

        if event.state & gtk.gdk.CONTROL_MASK \
        and event.state & gtk.gdk.SHIFT_MASK:
            shortcut = "ctrl+shift+" + unichr(event.keyval).lower()
            if shortcut in self.trigger_manager.shortcuts:
                r = self.trigger_manager.shortcuts[shortcut]()
                self.trigger_manager.last_shortcut = shortcut
                return r
            else:
                return False
        elif event.state & gtk.gdk.CONTROL_MASK:
            shortcut = "ctrl+" + unichr(event.keyval).lower()
            if shortcut in self.trigger_manager.shortcuts:
                r = self.trigger_manager.shortcuts[shortcut]()
                self.trigger_manager.last_shortcut = shortcut
                return r
            else:
                return False
        elif event.state & gtk.gdk.SHIFT_MASK:
            shortcut = "shift+" + unichr(event.keyval).lower()
            shortcut2 = unichr(event.keyval).lower()
            if shortcut in self.trigger_manager.shortcuts:
                r = self.trigger_manager.shortcuts[shortcut]()
                self.trigger_manager.last_shortcut = shortcut
                return r
            elif shortcut2 in self.trigger_manager.shortcuts:
                r = self.trigger_manager.shortcuts[shortcut2]()
                self.trigger_manager.last_shortcut = shortcut2
                return r
            else:
                return False
        else:
            shortcut = unichr(event.keyval).lower()
            if shortcut in self.trigger_manager.shortcuts:
                r = self.trigger_manager.shortcuts[shortcut]()
                self.trigger_manager.last_shortcut = shortcut
                return r
            else:
                return False
            
    def __add_icon_to_button(self, button):
        """
        Add the close image to a button.
        
        @param button: Reference to a button.
        @type file_uri: A Button object.
        """
        icon_box = gtk.HBox(False, 0)
        image = gtk.Image()
        image.set_from_stock(gtk.STOCK_CLOSE,gtk.ICON_SIZE_MENU)
        gtk.Button.set_relief(button, gtk.RELIEF_NONE)
        settings = gtk.Widget.get_settings(button)
        (w, h) = gtk.icon_size_lookup_for_settings(settings, gtk.ICON_SIZE_MENU)
        gtk.Widget.set_size_request(button, w + 0, h + 1)
        image.show()
        icon_box.pack_start(image, True, False, 0)
        button.add(icon_box)

        icon_box.show()
    
    def open_new_tab(self, text, file_uri="", encode="utf-8"):
        """
        Open a new tab in the DocumentManager
        
        @param text: Tab label text.
        @type text: A string.
        
        @param file_uri: A file that will be displayed in TFSourceView.
        @type file_uri: A string.
        """
        tab_box = gtk.HBox(False, 4)
        tab_label = gtk.Label(text)
        tab_button = gtk.Button()
        tab_button.set_name('tab.close_button')
        
        self.__add_icon_to_button(tab_button)
        
        tab_box.pack_start(tab_label, True, True, 0)
        tab_box.pack_start(tab_button, False, False, 0)
        tab_box.show_all()
        
        swindow = gtk.ScrolledWindow()
        swindow.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_ALWAYS)
        
        view = self.__create_gsv(file_uri, encode)
        swindow.add(view)
        swindow.show_all()
        
        child = self.get_nth_page(0)
        
        if child != None:
            v_child = child.get_child()
        
        if self.get_n_pages() == 1:
            if v_child.get_file_uri() == "" and v_child.get_text() == "":
                if view.get_file_uri() != "":
                    self.remove_tab(view, 0)
        
        new_page_num = self.page_num(swindow)
        
        self.append_page(swindow, tab_box)
        self.set_tab_reorderable(swindow, True)
        self.set_current_page(new_page_num)
        tab_button.connect('clicked', self.close_tab_button, swindow)
        view.buffer.connect('changed', self.buffer_change, view, tab_label)
        view.grab_focus()
        
    def buffer_change(self, widget, view, label):
        if not view.modified:
            text = label.get_text()
            label.set_text("*" + text)
            view.modified = True
        
        self.search_functions.all_search = False
        #start, end = self.search_functions.get_line_iters()
        start, end = view.get_line_iters()
        self.search_functions.update_matches(start, end)
        
    def close_tab(self, swindow):
        """
        Close a tab in the DocumentManager
        
        @param swindow: A ScrolledWindow.
        @type swindow: A ScrolledWindow object.
        """
        view = swindow.get_child()
        
        if not view.is_updated():
            dialog = CloseAlertDialog()
            
            button_clicked = dialog.run()
            
            if button_clicked == gtk.RESPONSE_DELETE_EVENT:
                dialog.destroy()
                return
            elif button_clicked == 1:
                dialog.destroy()
            elif button_clicked == 2:
                dialog.destroy()
                return
            elif button_clicked == 3:
                file_uri = view.get_file_uri()
                if file_uri != "":
                    self.save_file_tab(file_uri)
                else:
                    file_save = SaveFileDialog(constants.MESSAGE_0002, None,
                                               "", "*", False)
                    files = file_save.run()
                    file_save.destroy()
            
                    if len(files):
                        self.save_file_tab(files[0])
                
                dialog.destroy()
            
        #Close tab frees memory correctly?
        page = self.page_num(swindow)
        self.remove_tab(view, page)
        
        if self.get_n_pages() == 0:
            self.main_window.set_title(constants.MESSAGE_0015)
            self.interface.statusbar.clear()
        
    def remove_tab(self, view, page):
        view.word_complete.stop_indexer()
        self.remove_page(page)
    
    def close_tab_button(self, widget, swindow):
        """
        Close a tab in the DocumentManager
        
        @param widget: Reference to the widget.
        @type widget: A widget object.
        
        @param swindow: A ScrolledWindow.
        @type swindow: A ScrolledWindow object.
        """
        self.close_tab(swindow)
        
    def open_file_tab(self, new_file, encode="utf-8"):
        """
        Open a new tab with a file associated.
        
        @param new_file: A file that will be displayed in the new tab.
        @type new_file: A string.
        """
        self.open_new_tab(os.path.basename(new_file), new_file, encode)
    
    def save_file_tab(self, save_file):
        """
        Save the text of active tab in a file.
        
        @param save_file: A file that will be save.
        @type save_file: A string.
        """
        view = self.get_active_view()
        
        try:
            arquivo = open(save_file, "w")
        except IOError:
            error_dialog = ErrorDialog(constants.MESSAGE_0018,
                                       constants.MESSAGE_0019)
            button_clicked = error_dialog.run()
            
            if button_clicked == gtk.RESPONSE_DELETE_EVENT:
                error_dialog.destroy()
                return
            elif button_clicked == 1:
                error_dialog.destroy()
        else:
            arquivo.write(view.get_text().encode(view.encode))
            arquivo.close()
        
            if view.get_file_uri() != save_file:
                view.set_file_uri(save_file)
            
            current_page = self.get_current_page()
            current_child = self.get_nth_page(current_page)
            tab_label = self.get_tab_label(current_child)
            children = tab_label.get_children()
            
            f_list = save_file.split("/")
            file_name = f_list.pop()
            direc = " (" + "/".join(f_list) + ")"
            
            children[0].set_label(file_name)
            
            self.main_window.set_title(str(file_name) + direc)
    #        begin = view.buffer.get_start_iter()
    #        end = view.buffer.get_end_iter()
    #         self.search_functions.update_matches(begin, end)
            
            view.modified = False
    
    def get_active_view(self):
        """
        Get the active TFSourceView
        
        @return: The active TFSourceView.
        @rtype: A TFSourceView object.
        """
        current_page = self.get_current_page()
        if current_page == -1:
            return None
        else:
            return self.get_nth_page(current_page).get_child()
            
    def change_encode(self, encode):
        page_num = self.get_current_page()
        
        if page_num != -1:
            swindow = self.get_nth_page(page_num)
            view = swindow.get_child()

            if view.encode != encode:
                file_uri = view.file_uri
                
                if file_uri != "":
                    if not view.is_updated():
                        dialog = CloseAlertDialog()
                        dialog.set_markup(constants.MESSAGE_0014)

                        button_clicked = dialog.run()

                        if button_clicked == gtk.RESPONSE_DELETE_EVENT:
                            dialog.destroy()
                            return
                        elif button_clicked == 1:
                            dialog.destroy()
                        elif button_clicked == 2:
                            dialog.destroy()
                            return
                        elif button_clicked == 3:
                            self.save_file_tab(file_uri)
                           
                            dialog.destroy()
                                                       
                        page = self.page_num(swindow)
                        self.remove_tab(view, page)
                        self.open_file_tab(file_uri, encode)
                    else:
                        page = self.page_num(swindow)
                        self.remove_tab(view, page)
                        self.open_file_tab(file_uri, encode)
            
    def set_style(self, view, value):
        """
        Set the color style used by a TFSourceView.
        
        @param view: The TFSourceView.
        @type view: A TFSourceView object.
        
        @param value: The font name.
        @type value: A string.
        """
        
        scheme = self.style_manager.get_scheme(value)
        buffer = view.buffer
        
        if scheme != None:
            buffer.set_style_scheme(scheme)
            
            style_scheme = buffer.get_style_scheme()
            style = style_scheme.get_style("search-match")
            style_sticky = style_scheme.get_style("def:note")
            
            # Set search match color
            if style != None:
                foreground = style.get_property("foreground")
                background = style.get_property("background")
            else:
                foreground = "black"
                background = "lightblue"

            if style_sticky != None:
                foreground_sticky = style_sticky.get_property("foreground")
                background_sticky = style_sticky.get_property("background")
            else:
                foreground_sticky = "black"
                background_sticky = "#ff6100"
                
            tag_table = buffer.get_tag_table()
            tag = tag_table.lookup("searchmatch")
            tag_sticky = tag_table.lookup("sticky")
            
            if foreground != None:
                tag.set_property("foreground", foreground)
                tag_sticky.set_property("foreground", foreground_sticky)
            else:
                tag.set_property("foreground", "black")
                tag_sticky.set_property("foreground", "black")
            
            if background != None:
                tag.set_property("background", background)
                tag_sticky.set_property("background", background_sticky)
            
    def set_font(self, view, value):
        """
        Set the font used by a TFSourceView.
        
        @param view: The TFSourceView.
        @type view: A TFSourceView object.
        
        @param value: The font name.
        @type value: A string.
        """
        view.modify_font(pango.FontDescription(value))
        
    def set_tab_width(self, view, value):
        """
        Set the tab width of a TFSourceView.
        
        @param view: The TFSourceView.
        @type view: A TFSourceView object.
        
        @param value: The tab width.
        @type value: A integer.
        """
        view.set_tab_width(value)
        
    def set_use_spaces(self, view, value):
        """
        Set if the editor uses spaces instead of tabs or not in a TFSourceView.
        
        @param view: The TFSourceView.
        @type view: A TFSourceView object.
        
        @param value: True if the editor will use space instead tabs.
        @type value: A boolean.
        """
        view.set_insert_spaces_instead_of_tabs(value)
        
    def set_auto_indent(self, view, value):
        """
        Enable or disable auto indent in a TFSourceView.
        
        @param view: The TFSourceView.
        @type view: A TFSourceView object.
        
        @param value: True to enable auto indent and False to disable.
        @type value: A boolean.
        """
        view.set_auto_indent(value)
        
    def set_show_right_margin(self, view, value):
        """
        Enable or disable the right margin in a TFSourceView.
        
        @param view: The TFSourceView.
        @type view: A TFSourceView object.
        
        @param value: True to enable the right margin and False to disable.
        @type value: A boolean.
        """
        view.set_show_right_margin(value)
        
    def set_right_margin_position(self, view, value):
        """
        Set the right margin position in a TFSourceView.
        
        @param view: The TFSourceView.
        @type view: A TFSourceView object.
        
        @param value: The right margin position.
        @type value: A integer.
        """
        view.set_right_margin_position(value)
        
    def set_show_line_numbers(self, view, value):
        """
        Enable or disable the line numbers in a TFSourceView.
        
        @param view: The TFSourceView.
        @type view: A TFSourceView object.
        
        @param value: True to enable the line numbers and False to disable.
        @type value: A boolean.
        """
        view.set_show_line_numbers(value)
        
    def set_highlight_current_line(self, view, value):
        """
        Enable or disable the line numbers in a TFSourceView.
        
        @param view: The TFSourceView.
        @type view: A TFSourceView object.
        
        @param value: True to enable the line numbers and False to disable.
        @type value: A boolean.
        """
        view.set_highlight_current_line(value)
        
    def set_text_wrapping(self, view, value):
        """
        Enable or disable text wrapping in a TFSourceView.
        
        @param view: The TFSourceView.
        @type view: A TFSourceView object.
        
        @param value: True to enable text wrapping and False to disable.
        @type value: A boolean.
        """
        if value:
            view.set_wrap_mode(gtk.WRAP_WORD)
        else:
            view.set_wrap_mode(gtk.WRAP_NONE)
            
    def set_brackets_matching(self, view, value):
        """
        Enable or disable highlight for brackets matching in a TFSourceView.
        
        @param view: The TFSourceView.
        @type view: A TFSourceView object.
        
        @param value: True to enable highlight for brackets
        matching and False to disable.
        @type value: A boolean.
        """
        view.buffer.set_highlight_matching_brackets(value)
