# -*- coding: utf-8 -*-

#######################################################################
# Copyright © 2007 Yuri Malheiros.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published
# the Free Software Foundation; version 2 only.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#######################################################################

"""
This module implements a class responsible for creating the TFSourceView.
"""

import gtksourceview2
import codecs
import gnomevfs

from TextFlow.com.SnippetManager import SnippetManager
from TextFlow.com.WordComplete import WordComplete
import TextFlow.com.Files as Files

class TFSourceView(gtksourceview2.View):
    """
    This class implements the TFSourceView widget.
    """
    
    def __init__(self, buffer, file_uri="", encode="utf-8"):
        """
        Constructor.
        
        @param buffer: The SourceBuffer of this view.
        @type buffer: A SourceBuffer object.
        
        @param file_uri: The file that will be opened in this view.
        @type file_uri: A string.
        """
        super(TFSourceView, self).__init__(buffer)
        self.buffer = buffer
        self.file_uri = file_uri
        self.modified = False
        self.encode = encode
        self.snippets = SnippetManager(self)
        self.word_complete = WordComplete(self)
        
        if self.file_uri != "":
#            current_file = codecs.open(file_uri, "r", "utf-8", "replace")
            current_file = codecs.open(file_uri, "r", encode, "replace")
            self.buffer.begin_not_undoable_action()
            self.buffer.set_text(current_file.read().replace("\r",""))
            self.buffer.end_not_undoable_action()
            self.buffer.place_cursor(self.buffer.get_start_iter())
            current_file.close()
            
            self.__set_language(file_uri)
            self.word_complete.indexer()
            
    def __set_language(self, file_uri):
        lm = gtksourceview2.LanguageManager()    
        lang_id = Files.get_language_from_mime(gnomevfs.get_mime_type(file_uri))
        
        if lang_id != None:
            language = lm.get_language(lang_id)
            self.snippets.load(lang_id)
            self.buffer.set_language(language)
            
    def get_file_uri(self):
        """
        Get the uri of the file opened in this view.
        
        @return: The uri of the file opened in this view.
        @rtype: A string.
        """
        return self.file_uri
        
    def set_file_uri(self, new_file_uri):
        """
        Set the uri of the file opened in this view.
        
        @param new_file_uri: The new uri of the file opened.
        @type file_uri: A string.
        """
        self.file_uri = new_file_uri
        self.__set_language(new_file_uri)
        
    def get_text(self):
        """
        Get the text of the active view.
        
        @return: The text of the active view.
        @rtype: A string.
        """
        return self.buffer.get_text(self.buffer.get_start_iter(),
                                    self.buffer.get_end_iter(), True)
                                    
    def get_line_iters(self):
        """
        Get the iter at the start and at the end of a current line.
        
        @return: the iter at the start and the iter at the end of current line
        @rtype: A tuple.
        """
        buffer = self.buffer
        start_iter = buffer.get_iter_at_mark(buffer.get_insert())
        line_num = start_iter.get_line()
        end_iter = start_iter.copy()
        if not end_iter.ends_line():
            end_iter.forward_to_line_end()
        start_iter = buffer.get_iter_at_line(line_num)
        
        return (start_iter, end_iter)
    
    def get_indentation(self, insert_iter):
        """
        This method gets the current line indentation
        
        @param insert_iter: The insert cursor iter.
        @type insert_iter: A TextIter object.
        
        @return: A string of characters (tabs and/or spaces)
        @rtype: A string.
        """
        buffer = self.buffer
        line_num = insert_iter.get_line()
        line_iter = buffer.get_iter_at_line(line_num)
        indent = []
        while True:
            char = line_iter.get_char()
            if char == " " or char == "\t":
                line_iter.forward_char()
                indent.append(char)
            else:
                break
        indent = "".join(indent)
        
        return indent
    
    def get_cursor_position(self):
        cursor_mark = self.buffer.get_insert()
        cursor_iter = self.buffer.get_iter_at_mark(cursor_mark)
        
        cursor_line = cursor_iter.get_line()
        start_iterator = self.buffer.get_iter_at_line(cursor_line)

        line_text = self.buffer.get_text(start_iterator, cursor_iter)
        tabs_width = self.get_tab_width()
        count = 0
        
        for characters in line_text:
            if characters == "\t":
                count += (tabs_width - (count % tabs_width))
            else:
                count += 1
        cursor_column = count
        
        return cursor_line+1, cursor_column+1
    
    def is_updated(self):
        """
        Verify if the file is updated.
        """
        view_text = self.get_text()
        
        if self.file_uri == "":
            if view_text == "":
                return True
            else:
                return False
        else:
            try:
                current_file = codecs.open(self.file_uri, "r", "utf-8",
                                           "replace")
            except IOError:
                return False

            file_text = current_file.read()
            current_file.close()
            
            if view_text == file_text:
                return True
            else:
                return False
    
#    def set_encoding(self, encode):
#        """
#        Set text encode used in this view.
#        
#        @param encode: the new encode.
#        @type encode: a string.
#        """
#        self.encode = encode
#        start, end = self.buffer.get_bounds()
#        text = self.buffer.get_text(start, end)
#        self.buffer.delete(start, end)
#        #TODO: Tratar erro aqui
#        text = text.encode(self.encode, "replace")
#        self.buffer.set_text(text.decode(encode, "replace"))
#        
#        if self.file_uri != "":
#            current_file = codecs.open(self.file_uri, "r", encode, "replace")
#            self.buffer.begin_not_undoable_action()
#            self.buffer.set_text(current_file.read().replace("\r",""))
#            self.buffer.end_not_undoable_action()
#            self.buffer.place_cursor(self.buffer.get_start_iter())
#            current_file.close()
